/* $Id: makeimagedl.hh,v 1.3 2003/04/23 14:53:44 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2003 Richard Atterer
  | \/|  <richard at atterer.net>
   '` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Download .jigdo/.template and file URLs

*/

#ifndef MAKEIMAGEDL_HH
#define MAKEIMAGEDL_HH

#include <config.h>

#include <string>

#include <job.hh>
#include <makeimage.hh>
#include <single-url.hh>
//______________________________________________________________________

namespace Job {
  class MakeImageDl;
}

class Job::MakeImageDl {
public:
  class IO;
  IOPtr<IO> io;

  enum State {
    DOWNLOADING_JIGDO
  };

  /** Prepare object. No download is started yet.
      @param destination Name of tmp directory for storing data during the
      download. */
  MakeImageDl(IO* ioPtr, const string& jigdoURI, const string& destination);

  /** Start downloading. First creates a new download for the .jigdo data,
      then the .template data, etc. */
  void run();

  inline const string& jigdoURI() const;

  /** Return current state of object. */
  inline State state() const;

private:
  // Wraps around a SingleURL, for downloading the .jigdo file
  struct JigdoDownload;

  // Don't copy
  inline MakeImageDl& operator=(const MakeImageDl&);

  // State, e.g. "downloading jigdo file", "error"
  State stateVal;

  // URL of .jigdo file, corresponding download handler (null if finished)
  string jigdoURL;
  JigdoDownload* jigdo;

  string dest; // Destination dir

  // Workhorse which actually generates the image from the data we feed it
  MakeImage mi;
};
//______________________________________________________________________

/** User interaction for MakeImageDl. */
class Job::MakeImageDl::IO : public Job::IO {
public:

  /** Called by the job when it is deleted or when a different IO object is
      registered with it. If the IO object considers itself owned by its job,
      it can delete itself. */
  virtual void job_deleted() = 0;

  /** Called when the job has successfully completed its task. */
  virtual void job_succeeded() = 0;

  /** Called when the job fails. The only remaining action after getting this
      is to delete the job object. */
  virtual void job_failed(string* message) = 0;

  /** Informational message. */
  virtual void job_message(string* message) = 0;

  /** Called by MakeImageDl after it has started a new download; either
      the download of the .jigdo file or the .template file, or the download
      of a part. The GTK+ GUI uses this to display the new SingleURL as a
      "child" of the MakeImageDl.
      @return IO object to associate with this child download. Anything
      happening to the SingleURL child will be passed on to the object you
      return here. Can return null to prevent this. */
  virtual Job::SingleURL::IO* makeImageDl_new(Job::SingleURL* childDownload)
    = 0;

  /** A child has (successfully or not) finished. */
  virtual void makeImageDl_finished(Job::SingleURL* childDownload) = 0;
};
//======================================================================

// PRIVATE stuff, internal to MakeImageDl

/** Private class of MakeImageDl: Object encapsulating a SingleURL, which is
    used to retrieve the data of the .jigdo file. We need an extra object for
    this because its virtual methods are called by the
    SingleURL/JigdoConfig. */
struct Job::MakeImageDl::JigdoDownload
    : JigdoConfig::ProgressReporter, SingleURL::IO {
  JigdoDownload(MakeImageDl* m, const string& jigdoURL);
  virtual ~JigdoDownload();

  /** Methods from JigdoConfig::ProgressReporter */
  virtual void error(const string& message);
  virtual void info(const string& message);

  /** Methods from SingleURL::IO */
  virtual void job_deleted();
  virtual void job_succeeded();
  virtual void job_failed(string* message);
  virtual void job_message(string* message);
  virtual void singleURL_dataSize(uint64 n);
  virtual void singleURL_data(const byte* data, size_t size,
                              uint64 currentSize);

  MakeImageDl* master; // Ptr to the object which owns us
  IOPtr<Job::SingleURL::IO> io; // IO for this SingleURL, supplied by master
  SingleURL download; // Download object
};
//______________________________________________________________________

const string& Job::MakeImageDl::jigdoURI() const { return jigdoURL; }

#endif
